Moved inline functions from rsync.h into ifuncs.h.
[rsync/rsync.git] / util.c
diff --git a/util.c b/util.c
index 647d6f5..7a32025 100644 (file)
--- a/util.c
+++ b/util.c
@@ -4,11 +4,11 @@
  * Copyright (C) 1996-2000 Andrew Tridgell
  * Copyright (C) 1996 Paul Mackerras
  * Copyright (C) 2001, 2002 Martin Pool <mbp@samba.org>
- * Copyright (C) 2003, 2004, 2005, 2006 Wayne Davison
+ * Copyright (C) 2003-2007 Wayne Davison
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
+ * with this program; if not, visit the http://fsf.org website.
  */
 
 #include "rsync.h"
+#include "ifuncs.h"
 
 extern int verbose;
 extern int dry_run;
@@ -29,6 +29,7 @@ extern int module_id;
 extern int modify_window;
 extern int relative_paths;
 extern int human_readable;
+extern char *module_dir;
 extern unsigned int module_dirlen;
 extern mode_t orig_umask;
 extern char *partial_dir;
@@ -90,9 +91,9 @@ int fd_pair(int fd[2])
        return ret;
 }
 
-void print_child_argv(char **cmd)
+void print_child_argv(const char *prefix, char **cmd)
 {
-       rprintf(FCLIENT, "opening connection using ");
+       rprintf(FCLIENT, "%s ", prefix);
        for (; *cmd; cmd++) {
                /* Look for characters that ought to be quoted.  This
                * is not a great quoting algorithm, but it's
@@ -519,8 +520,7 @@ static int filter_server_path(char *arg)
        return 0;
 }
 
-static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
-                           int *maxargs_ptr)
+void glob_expand(char *s, char ***argv_ptr, int *argc_ptr, int *maxargs_ptr)
 {
        char **argv = *argv_ptr;
        int argc = *argc_ptr;
@@ -529,7 +529,7 @@ static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
        if (argc == maxargs) {
                maxargs += MAX_ARGS;
                if (!(argv = realloc_array(argv, char *, maxargs)))
-                       out_of_memory("glob_expand_one");
+                       out_of_memory("glob_expand");
                *argv_ptr = argv;
                *maxargs_ptr = maxargs;
        }
@@ -549,6 +549,8 @@ static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
                s = sanitize_path(NULL, s, "", 0, NULL);
        else
                s = strdup(s);
+       if (!s)
+               out_of_memory("glob_expand");
 
        memset(&globbuf, 0, sizeof globbuf);
        if (!filter_server_path(s))
@@ -556,7 +558,7 @@ static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
        if (MAX((int)globbuf.gl_pathc, 1) > maxargs - argc) {
                maxargs += globbuf.gl_pathc + MAX_ARGS;
                if (!(argv = realloc_array(argv, char *, maxargs)))
-                       out_of_memory("glob_expand_one");
+                       out_of_memory("glob_expand");
                *argv_ptr = argv;
                *maxargs_ptr = maxargs;
        }
@@ -567,7 +569,7 @@ static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
                free(s);
                for (i = 0; i < (int)globbuf.gl_pathc; i++) {
                        if (!(argv[argc++] = strdup(globbuf.gl_pathv[i])))
-                               out_of_memory("glob_expand_one");
+                               out_of_memory("glob_expand");
                }
        }
        globfree(&globbuf);
@@ -576,35 +578,34 @@ static void glob_expand_one(char *s, char ***argv_ptr, int *argc_ptr,
 }
 
 /* This routine is only used in daemon mode. */
-void glob_expand(char *base1, char ***argv_ptr, int *argc_ptr, int *maxargs_ptr)
+void glob_expand_module(char *base1, char *arg, char ***argv_ptr, int *argc_ptr, int *maxargs_ptr)
 {
-       char *s = (*argv_ptr)[*argc_ptr];
-       char *p, *q;
+       char *p, *s;
        char *base = base1;
        int base_len = strlen(base);
 
-       if (!s || !*s)
+       if (!arg || !*arg)
                return;
 
-       if (strncmp(s, base, base_len) == 0)
-               s += base_len;
+       if (strncmp(arg, base, base_len) == 0)
+               arg += base_len;
 
-       if (!(s = strdup(s)))
-               out_of_memory("glob_expand");
+       if (!(arg = strdup(arg)))
+               out_of_memory("glob_expand_module");
 
        if (asprintf(&base," %s/", base1) <= 0)
-               out_of_memory("glob_expand");
+               out_of_memory("glob_expand_module");
        base_len++;
 
-       for (q = s; *q; q = p + base_len) {
-               if ((p = strstr(q, base)) != NULL)
+       for (s = arg; *s; s = p + base_len) {
+               if ((p = strstr(s, base)) != NULL)
                        *p = '\0'; /* split it at this point */
-               glob_expand_one(q, argv_ptr, argc_ptr, maxargs_ptr);
+               glob_expand(s, argv_ptr, argc_ptr, maxargs_ptr);
                if (!p)
                        break;
        }
 
-       free(s);
+       free(arg);
        free(base);
 }
 
@@ -750,7 +751,7 @@ unsigned int clean_fname(char *name, BOOL collapse_dot_dot)
  * rootdir will be ignored to avoid expansion of the string.
  *
  * The rootdir string contains a value to use in place of a leading slash.
* Specify NULL to get the default of lp_path(module_id).
  *
  * The depth var is a count of how many '..'s to allow at the start of the
  * path.  If symlink is set, combine its value with the "p" value to get
@@ -779,7 +780,7 @@ char *sanitize_path(char *dest, const char *p, const char *rootdir, int depth,
                int plen = strlen(p);
                if (*p == '/') {
                        if (!rootdir)
-                               rootdir = lp_path(module_id);
+                               rootdir = module_dir;
                        rlen = strlen(rootdir);
                        depth = 0;
                        p++;
@@ -884,8 +885,10 @@ int push_dir(const char *dir, int set_path_only)
        if (len == 1 && *dir == '.')
                return 1;
 
-       if ((*dir == '/' ? len : curr_dir_len + 1 + len) >= sizeof curr_dir)
+       if ((*dir == '/' ? len : curr_dir_len + 1 + len) >= sizeof curr_dir) {
+               errno = ENAMETOOLONG;
                return 0;
+       }
 
        if (!set_path_only && chdir(dir))
                return 0;
@@ -1253,11 +1256,11 @@ int _Insure_trap_error(int a1, int a2, int a3, int a4, int a5, int a6)
 
 #define MALLOC_MAX 0x40000000
 
-void *_new_array(unsigned int size, unsigned long num)
+void *_new_array(unsigned long num, unsigned int size, int use_calloc)
 {
        if (num >= MALLOC_MAX/size)
                return NULL;
-       return malloc(size * num);
+       return use_calloc ? calloc(num, size) : malloc(num * size);
 }
 
 void *_realloc_array(void *ptr, unsigned int size, unsigned long num)
@@ -1467,3 +1470,31 @@ int bitbag_next_bit(struct bitbag *bb, int after)
 
        return -1;
 }
+
+void *expand_item_list(item_list *lp, size_t item_size,
+                      const char *desc, int incr)
+{
+       /* First time through, 0 <= 0, so list is expanded. */
+       if (lp->malloced <= lp->count) {
+               void *new_ptr;
+               size_t new_size = lp->malloced;
+               if (incr < 0)
+                       new_size += -incr; /* increase slowly */
+               else if (new_size < (size_t)incr)
+                       new_size += incr;
+               else
+                       new_size *= 2;
+               new_ptr = realloc_array(lp->items, char, new_size * item_size);
+               if (verbose >= 4) {
+                       rprintf(FINFO, "[%s] expand %s to %.0f bytes, did%s move\n",
+                               who_am_i(), desc, (double)new_size * item_size,
+                               new_ptr == lp->items ? " not" : "");
+               }
+               if (!new_ptr)
+                       out_of_memory("expand_item_list");
+
+               lp->items = new_ptr;
+               lp->malloced = new_size;
+       }
+       return (char*)lp->items + (lp->count++ * item_size);
+}
@@ -750,7 +751,7 @@ unsigned int clean_fname(char *name, BOOL collapse_dot_dot)
  *
  * The rootdir string contains a value to use in place of a leading slash.
- * Specify NULL to get the default of lp_path(module_id).
+ * Specify NULL to get the default of "module_dir".
  *
  * The depth var is a count of how many '..'s to allow at the start of the